﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Sharp1NoAdvanced : MonoBehaviour
{
    // ※初めての人向けではない内容を含んでいます！
    void Start()
    {
        Variable();
        Increment();
        IfEx();
        IfShortCircuit();
        IfNoGood();
        MaxMin();
    }

    void Variable()
    {
        // 以下の3つは全部同じ int型の変数
        int i1 = 11;

        // int の正体は System.Int32 であり、intはただのエイリアス(別名)
        System.Int32 i2 = 22;

        // var は型推論 (右辺より int と定義される)
        var i3 = 33;

        // 未使用の変数は、リリースビルドの最適化で消える
        var i4 = i1 + i2 + i3;
    }

    void Increment()
    {
        // 後置インクリメント
        int i10 = 100;
        int i11 = 1;
        i10 += i11++;

        // 前置インクリメント
        int i20 = 100;
        int i21 = 1;
        i20 += ++i21;

        var strings = new[] { "Name", "Level", "Hp", "Mp" };

        // インクリメントを活用して配列にアクセス
        var i = 0;
        var name = strings[i++];
        var level = strings[i++];
        var hp = strings[i++];
        var mp = strings[i];
    }

    void IfEx()
    {
        int i = 10;
        bool b = false;

        // if の後の { } がなくても良い
        if (i == 10)
            b = true;  // if の次の1行のみ有効
        else
            b = false; // else の次の1行のみ有効

        // 以下の3つは全部同じ

        if (i == 10)
        {
            b = true;
        }

        if (i == 10)
            b = true;

        if (i == 10) b = true;
    }

    void IfShortCircuit()
    {
        int i = 10;

        // ショートサーキット(短絡評価)
        // || ではTrueが確定した時点で、後続の条件判定を行わない
        if (i == 1 || Is10(i))
        {
            // Is10(i) は実行されない
            bool b = true;
        }

        // | で評価すると、後続の条件判定も行う
        if (i == 1 | Is10(i))
        {
            // Is10(i) が実行されるし、Is10(i)がfalseでも関係ない
            bool b = true;
        }

        // バグの原因や性能低下に繋がるので、必ず || && で書く！
    }

    void IfNoGood()
    {
        int i = 10;

        if (Is10(i) == true) // == trueは要らない
        {
        }
        // これと同じことを書いてる
        if (i == 10 == true)
        {
        }

        if (Is10(i = 11)) // i が 11に書き換わっている
        {
        }
        else
        { // ここにブレイクポイントで止まる
        }

        // ヨーダ記法は不要 (左側に定数を書く記法)
        if (10 == i)
        {
            // C#はifの中に bool しか書けないので問題ない
        }

        bool b = i == 999;

        // bool だとifの中で代入できてしまうが、警告される
        if (b = true)
        {

        }
    }

    bool Is10(int i)
    {
        return i == 10;
    }

    void MaxMin()
    {
        var max = int.MaxValue;
        var min = int.MinValue;

        max++;
        min--;

        var max2 = int.MaxValue;

        checked
        {
            // オーバーフローで例外発生
            max2++;
        }
    }

    // ContextMenu属性をつけると、Inspectorビューの Sharp1NoAdvancedを右クリックして、
    // "おまけ"というメニューから Omake() メソッドを実行できる。
    [ContextMenu("おまけ")]
    void Omake()
    {
        for (int i = 0; i < 5; i++)
        {
            for (int j = 0; j < 5; j++)
            {
                // キューブを作る
                var cube = GameObject.CreatePrimitive(PrimitiveType.Cube);

                // 移動
                cube.transform.Translate(i * 2, j * 2, 2);

                // 重力を与える
                cube.AddComponent<Rigidbody>();
            }
        }
    }
}